@@ -1,7 +1,6 @@
 /* Definitions of target machine for GNU compiler,
    for Alpha NetBSD systems.
-   Copyright (C) 1998, 2002, 2003, 2004, 2005, 2007
-   Free Software Foundation, Inc.
+   Copyright (C) 1998-2013 Free Software Foundation, Inc.
 
 This file is part of GCC.
 
@@ -19,9 +18,6 @@ You should have received a copy of the GNU General Public License
 along with GCC; see the file COPYING3.  If not see
 <http://www.gnu.org/licenses/>.  */
 
-#undef TARGET_DEFAULT
-#define TARGET_DEFAULT (MASK_FPREGS | MASK_GAS)
-
 #define TARGET_OS_CPP_BUILTINS()		\
     do {					\
 	NETBSD_OS_CPP_BUILTINS_ELF();		\
@@ -38,14 +34,14 @@ along with GCC; see the file COPYING3.  If not see
 #define TARGET_PROFILING_NEEDS_GP 1
 
 
-/* Provide a CPP_SUBTARGET_SPEC appropriate for NetBSD/alpha.  We use
+/* Provide a CPP_SPEC appropriate for NetBSD/alpha.  We use
    this to pull in CPP specs that all NetBSD configurations need.  */
 
-#undef CPP_SUBTARGET_SPEC
-#define CPP_SUBTARGET_SPEC NETBSD_CPP_SPEC
+#undef CPP_SPEC
+#define CPP_SPEC NETBSD_CPP_SPEC
 
-#undef SUBTARGET_EXTRA_SPECS
-#define SUBTARGET_EXTRA_SPECS			\
+#undef EXTRA_SPECS
+#define EXTRA_SPECS			\
   { "netbsd_link_spec", NETBSD_LINK_SPEC_ELF },	\
   { "netbsd_entry_point", NETBSD_ENTRY_POINT },	\
   { "netbsd_endfile_spec", NETBSD_ENDFILE_SPEC },
@@ -70,14 +66,7 @@ along with GCC; see the file COPYING3.  If not see
 
 #undef ENDFILE_SPEC
 #define ENDFILE_SPEC		\
-  "%{ffast-math|funsafe-math-optimizations:crtfm%O%s} \
+  "%{Ofast|ffast-math|funsafe-math-optimizations:crtfm%O%s} \
    %(netbsd_endfile_spec)"
 
-
-/* Attempt to enable execute permissions on the stack.  */
-
-#define ENABLE_EXECUTE_STACK NETBSD_ENABLE_EXECUTE_STACK
-
-
-#undef TARGET_VERSION
-#define TARGET_VERSION fprintf (stderr, " (NetBSD/alpha ELF)");
+#define HAVE_ENABLE_EXECUTE_STACK
